@@ -1,5 +1,10 @@
 package Monitoring::GLPlugin;
-# ABSTRACT: helper functions to build a upnp-based monitoring plugin
+
+=head1 Monitoring::GLPlugin 
+
+Monitoring::GLPlugin - infrastructure functions to build a monitoring plugin
+
+=cut
 
 use strict;
 use IO::File;
@@ -7,7 +12,7 @@ use File::Basename;
 use Digest::MD5 qw(md5_hex);
 use Errno;
 our $AUTOLOAD;
-our $VERSION = "1.0";
+our $VERSION = "1.1.1";
 
 use constant { OK => 0, WARNING => 1, CRITICAL => 2, UNKNOWN => 3 };
 
@@ -27,6 +32,12 @@ sub new {
   my %params = @_;
   my $self = {};
   bless $self, $class;
+  require Monitoring::GLPlugin::Commandline
+      if ! grep /BEGIN/, keys %Monitoring::GLPlugin::Commandline::;
+  require Monitoring::GLPlugin::Item
+      if ! grep /BEGIN/, keys %Monitoring::GLPlugin::Item::;
+  require Monitoring::GLPlugin::TableItem
+      if ! grep /BEGIN/, keys %Monitoring::GLPlugin::TableItem::;
   $Monitoring::GLPlugin::plugin = Monitoring::GLPlugin::Commandline->new(%params);
   return $self;
 }
@@ -1335,801 +1346,6 @@ sub AUTOLOAD {
   }
 }
 
+1;
 
-package Monitoring::GLPlugin::Commandline;
-use strict;
-use constant { OK => 0, WARNING => 1, CRITICAL => 2, UNKNOWN => 3, DEPENDENT => 4 };
-our %ERRORS = (
-    'OK'        => OK,
-    'WARNING'   => WARNING,
-    'CRITICAL'  => CRITICAL,
-    'UNKNOWN'   => UNKNOWN,
-    'DEPENDENT' => DEPENDENT,
-);
-
-our %STATUS_TEXT = reverse %ERRORS;
-
-
-sub new {
-  my $class = shift;
-  my %params = @_;
-  my $self = {
-       perfdata => [],
-       messages => {
-         ok => [],
-         warning => [],
-         critical => [],
-         unknown => [],
-       },
-       args => [],
-       opts => Monitoring::GLPlugin::Commandline::Getopt->new(%params),
-       modes => [],
-       statefilesdir => undef,
-  };
-  foreach (qw(shortname usage version url plugin blurb extra
-      license timeout)) {
-    $self->{$_} = $params{$_};
-  }
-  bless $self, $class;
-  $self->{plugin} ||= $Monitoring::GLPlugin::pluginname;
-  $self->{name} = $self->{plugin};
-  $Monitoring::GLPlugin::plugin = $self;
-}
-
-sub AUTOLOAD {
-  my $self = shift;
-  return if ($AUTOLOAD =~ /DESTROY/);
-  $self->debug("AUTOLOAD %s\n", $AUTOLOAD)
-        if $self->{opts}->verbose >= 2;
-  if ($AUTOLOAD =~ /^.*::(add_arg|override_opt|create_opt)$/) {
-    $self->{opts}->$1(@_);
-  }
-}
-
-sub DESTROY {
-  my $self = shift;
-  # ohne dieses DESTROY rennt nagios_exit in obiges AUTOLOAD rein
-  # und fliegt aufs Maul, weil {opts} bereits nicht mehr existiert.
-  # Unerklaerliches Verhalten.
-}
-
-sub debug {
-  my $self = shift;
-  my $format = shift;
-  my $tracefile = "/tmp/".$Monitoring::GLPlugin::pluginname.".trace";
-  $self->{trace} = -f $tracefile ? 1 : 0;
-  if ($self->opts->verbose && $self->opts->verbose > 10) {
-    printf("%s: ", scalar localtime);
-    printf($format, @_);
-    printf "\n";
-  }
-  if ($self->{trace}) {
-    my $logfh = new IO::File;
-    $logfh->autoflush(1);
-    if ($logfh->open($tracefile, "a")) {
-      $logfh->printf("%s: ", scalar localtime);
-      $logfh->printf($format, @_);
-      $logfh->printf("\n");
-      $logfh->close();
-    }
-  }
-}
-
-sub opts {
-  my $self = shift;
-  return $self->{opts};
-}
-
-sub getopts {
-  my $self = shift;
-  $self->opts->getopts();
-}
-
-sub add_message {
-  my $self = shift;
-  my ($code, @messages) = @_;
-  $code = (qw(ok warning critical unknown))[$code] if $code =~ /^\d+$/;
-  $code = lc $code;
-  push @{$self->{messages}->{$code}}, @messages;
-}
-
-sub selected_perfdata {
-  my $self = shift;
-  my $label = shift;
-  if ($self->opts->can("selectedperfdata") && $self->opts->selectedperfdata) {
-    my $pattern = $self->opts->selectedperfdata;
-    return ($label =~ /$pattern/i) ? 1 : 0;
-  } else {
-    return 1;
-  }
-}
-
-sub add_perfdata {
-  my ($self, %args) = @_;
-#printf "add_perfdata %s\n", Data::Dumper::Dumper(\%args);
-#printf "add_perfdata %s\n", Data::Dumper::Dumper($self->{thresholds});
-#
-# wenn warning, critical, dann wird von oben ein expliziter wert mitgegeben
-# wenn thresholds
-#  wenn label in 
-#    warningx $self->{thresholds}->{$label}->{warning} existiert
-#  dann nimm $self->{thresholds}->{$label}->{warning}
-#  ansonsten thresholds->default->warning
-#
-
-  my $label = $args{label};
-  my $value = $args{value};
-  my $uom = $args{uom} || "";
-  my $format = '%d';
-
-  if ($self->opts->can("morphperfdata") && $self->opts->morphperfdata) {
-    # 'Intel [R] Interface (\d+) usage'='nic$1'
-    foreach my $key (keys %{$self->opts->morphperfdata}) {
-      if ($label =~ /$key/) {
-        my $replacement = '"'.$self->opts->morphperfdata->{$key}.'"';
-        my $oldlabel = $label;
-        $label =~ s/$key/$replacement/ee;
-        if (exists $self->{thresholds}->{$oldlabel}) {
-          %{$self->{thresholds}->{$label}} = %{$self->{thresholds}->{$oldlabel}};
-        }
-      }
-    }
-  }
-  if ($value =~ /\./) {
-    if (defined $args{places}) {
-      $value = sprintf '%.'.$args{places}.'f', $value;
-    } else {
-      $value = sprintf "%.2f", $value;
-    }
-  } else {
-    $value = sprintf "%d", $value;
-  }
-  my $warn = "";
-  my $crit = "";
-  my $min = defined $args{min} ? $args{min} : "";
-  my $max = defined $args{max} ? $args{max} : "";
-  if ($args{thresholds} || (! exists $args{warning} && ! exists $args{critical})) {
-    if (exists $self->{thresholds}->{$label}->{warning}) {
-      $warn = $self->{thresholds}->{$label}->{warning};
-    } elsif (exists $self->{thresholds}->{default}->{warning}) {
-      $warn = $self->{thresholds}->{default}->{warning};
-    }
-    if (exists $self->{thresholds}->{$label}->{critical}) {
-      $crit = $self->{thresholds}->{$label}->{critical};
-    } elsif (exists $self->{thresholds}->{default}->{critical}) {
-      $crit = $self->{thresholds}->{default}->{critical};
-    }
-  } else {
-    if ($args{warning}) {
-      $warn = $args{warning};
-    }
-    if ($args{critical}) {
-      $crit = $args{critical};
-    }
-  }
-  if ($uom eq "%") {
-    $min = 0;
-    $max = 100;
-  }
-  if (defined $args{places}) {
-    # cut off excessive decimals which may be the result of a division
-    # length = places*2, no trailing zeroes
-    if ($warn ne "") {
-      $warn = join("", map {
-          s/\.0+$//; $_
-      } map {
-          s/(\.[1-9]+)0+$/$1/; $_
-      } map {
-          /[\+\-\d\.]+/ ? sprintf '%.'.2*$args{places}.'f', $_ : $_;
-      } split(/([\+\-\d\.]+)/, $warn));
-    }
-    if ($crit ne "") {
-      $crit = join("", map {
-          s/\.0+$//; $_
-      } map {
-          s/(\.[1-9]+)0+$/$1/; $_
-      } map {
-          /[\+\-\d\.]+/ ? sprintf '%.'.2*$args{places}.'f', $_ : $_;
-      } split(/([\+\-\d\.]+)/, $crit));
-    }
-    if ($min ne "") {
-      $min = join("", map {
-          s/\.0+$//; $_
-      } map {
-          s/(\.[1-9]+)0+$/$1/; $_
-      } map {
-          /[\+\-\d\.]+/ ? sprintf '%.'.2*$args{places}.'f', $_ : $_;
-      } split(/([\+\-\d\.]+)/, $min));
-    }
-    if ($max ne "") {
-      $max = join("", map {
-          s/\.0+$//; $_
-      } map {
-          s/(\.[1-9]+)0+$/$1/; $_
-      } map {
-          /[\+\-\d\.]+/ ? sprintf '%.'.2*$args{places}.'f', $_ : $_;
-      } split(/([\+\-\d\.]+)/, $max));
-    }
-  }
-  push @{$self->{perfdata}}, sprintf("'%s'=%s%s;%s;%s;%s;%s",
-      $label, $value, $uom, $warn, $crit, $min, $max)
-      if $self->selected_perfdata($label);
-}
-
-sub add_html {
-  my $self = shift;
-  my $line = shift;
-  push @{$self->{html}}, $line;
-}
-
-sub suppress_messages {
-  my $self = shift;
-  $self->{suppress_messages} = 1;
-}
-
-sub clear_messages {
-  my $self = shift;
-  my $code = shift;
-  $code = (qw(ok warning critical unknown))[$code] if $code =~ /^\d+$/;
-  $code = lc $code;
-  $self->{messages}->{$code} = [];
-}
-
-sub reduce_messages_short {
-  my $self = shift;
-  my $message = shift || "no problems";
-  if ($self->opts->report && $self->opts->report eq "short") {
-    $self->clear_messages(OK);
-    $self->add_message(OK, $message) if ! $self->check_messages();
-  }
-}
-
-sub reduce_messages {
-  my $self = shift;
-  my $message = shift || "no problems";
-  $self->clear_messages(OK);
-  $self->add_message(OK, $message) if ! $self->check_messages();
-}
-
-sub check_messages {
-  my $self = shift;
-  my %args = @_;
-
-  # Add object messages to any passed in as args
-  for my $code (qw(critical warning unknown ok)) {
-    my $messages = $self->{messages}->{$code} || [];
-    if ($args{$code}) {
-      unless (ref $args{$code} eq 'ARRAY') {
-        if ($code eq 'ok') {
-          $args{$code} = [ $args{$code} ];
-        }
-      }
-      push @{$args{$code}}, @$messages;
-    } else {
-      $args{$code} = $messages;
-    }
-  }
-  my %arg = %args;
-  $arg{join} = ' ' unless defined $arg{join};
-
-  # Decide $code
-  my $code = OK;
-  $code ||= CRITICAL  if @{$arg{critical}};
-  $code ||= WARNING   if @{$arg{warning}};
-  $code ||= UNKNOWN   if @{$arg{unknown}};
-  return $code unless wantarray;
-
-  # Compose message
-  my $message = '';
-  if ($arg{join_all}) {
-      $message = join( $arg{join_all},
-          map { @$_ ? join( $arg{'join'}, @$_) : () }
-              $arg{critical},
-              $arg{warning},
-              $arg{unknown},
-              $arg{ok} ? (ref $arg{ok} ? $arg{ok} : [ $arg{ok} ]) : []
-      );
-  }
-
-  else {
-      $message ||= join( $arg{'join'}, @{$arg{critical}} )
-          if $code == CRITICAL;
-      $message ||= join( $arg{'join'}, @{$arg{warning}} )
-          if $code == WARNING;
-      $message ||= join( $arg{'join'}, @{$arg{unknown}} )
-          if $code == UNKNOWN;
-      $message ||= ref $arg{ok} ? join( $arg{'join'}, @{$arg{ok}} ) : $arg{ok}
-          if $arg{ok};
-  }
-
-  return ($code, $message);
-}
-
-sub status_code {
-  my $self = shift;
-  my $code = shift;
-  $code = (qw(ok warning critical unknown))[$code] if $code =~ /^\d+$/;
-  $code = uc $code;
-  $code = $ERRORS{$code} if defined $code && exists $ERRORS{$code};
-  $code = UNKNOWN unless defined $code && exists $STATUS_TEXT{$code};
-  return "$STATUS_TEXT{$code}";
-}
-
-sub perfdata_string {
-  my $self = shift;
-  if (scalar (@{$self->{perfdata}})) {
-    return join(" ", @{$self->{perfdata}});
-  } else {
-    return "";
-  }
-}
-
-sub html_string {
-  my $self = shift;
-  if (scalar (@{$self->{html}})) {
-    return join(" ", @{$self->{html}});
-  } else {
-    return "";
-  }
-}
-
-sub nagios_exit {
-  my $self = shift;
-  my ($code, $message, $arg) = @_;
-  $code = $ERRORS{$code} if defined $code && exists $ERRORS{$code};
-  $code = UNKNOWN unless defined $code && exists $STATUS_TEXT{$code};
-  $message = '' unless defined $message;
-  if (ref $message && ref $message eq 'ARRAY') {
-      $message = join(' ', map { chomp; $_ } @$message);
-  } else {
-      chomp $message;
-  }
-  if ($self->opts->negate) {
-    my $original_code = $code;
-    foreach my $from (keys %{$self->opts->negate}) {
-      if ((uc $from) =~ /^(OK|WARNING|CRITICAL|UNKNOWN)$/ &&
-          (uc $self->opts->negate->{$from}) =~ /^(OK|WARNING|CRITICAL|UNKNOWN)$/) {
-        if ($original_code == $ERRORS{uc $from}) {
-          $code = $ERRORS{uc $self->opts->negate->{$from}};
-        }
-      }
-    }
-  }
-  my $output = "$STATUS_TEXT{$code}";
-  $output .= " - $message" if defined $message && $message ne '';
-  if ($self->opts->can("morphmessage") && $self->opts->morphmessage) {
-    # 'Intel [R] Interface (\d+) usage'='nic$1'
-    # '^OK.*'="alles klar"   '^CRITICAL.*'="alles hi"
-    foreach my $key (keys %{$self->opts->morphmessage}) {
-      if ($output =~ /$key/) {
-        my $replacement = '"'.$self->opts->morphmessage->{$key}.'"';
-        $output =~ s/$key/$replacement/ee;
-      }
-    }
-  }
-  if (scalar (@{$self->{perfdata}})) {
-    $output .= " | ".$self->perfdata_string();
-  }
-  $output .= "\n";
-  if ($self->opts->can("isvalidtime") && ! $self->opts->isvalidtime) {
-    $code = OK;
-    $output = "OK - outside valid timerange. check results are not relevant now. original message was: ".
-        $output;
-  }
-  if (! exists $self->{suppress_messages}) {
-    print $output;
-  }
-  exit $code;
-}
-
-sub set_thresholds {
-  my $self = shift;
-  my %params = @_;
-  if (exists $params{metric}) {
-    my $metric = $params{metric};
-    # erst die hartcodierten defaultschwellwerte
-    $self->{thresholds}->{$metric}->{warning} = $params{warning};
-    $self->{thresholds}->{$metric}->{critical} = $params{critical};
-    # dann die defaultschwellwerte von der kommandozeile
-    if (defined $self->opts->warning) {
-      $self->{thresholds}->{$metric}->{warning} = $self->opts->warning;
-    }
-    if (defined $self->opts->critical) {
-      $self->{thresholds}->{$metric}->{critical} = $self->opts->critical;
-    }
-    # dann die ganz spezifischen schwellwerte von der kommandozeile
-    if ($self->opts->warningx) { # muss nicht auf defined geprueft werden, weils ein hash ist
-      foreach my $key (keys %{$self->opts->warningx}) {
-        next if $key ne $metric;
-        $self->{thresholds}->{$metric}->{warning} = $self->opts->warningx->{$key};
-      }
-    }
-    if ($self->opts->criticalx) {
-      foreach my $key (keys %{$self->opts->criticalx}) {
-        next if $key ne $metric;
-        $self->{thresholds}->{$metric}->{critical} = $self->opts->criticalx->{$key};
-      }
-    }
-  } else {
-    $self->{thresholds}->{default}->{warning} =
-        defined $self->opts->warning ? $self->opts->warning : defined $params{warning} ? $params{warning} : 0;
-    $self->{thresholds}->{default}->{critical} =
-        defined $self->opts->critical ? $self->opts->critical : defined $params{critical} ? $params{critical} : 0;
-  }
-}
-
-sub force_thresholds {
-  my $self = shift;
-  my %params = @_;
-  if (exists $params{metric}) {
-    my $metric = $params{metric};
-    $self->{thresholds}->{$metric}->{warning} = $params{warning} || 0;
-    $self->{thresholds}->{$metric}->{critical} = $params{critical} || 0;
-  } else {
-    $self->{thresholds}->{default}->{warning} = $params{warning} || 0;
-    $self->{thresholds}->{default}->{critical} = $params{critical} || 0;
-  }
-}
-
-sub get_thresholds {
-  my $self = shift;
-  my @params = @_;
-  if (scalar(@params) > 1) {
-    my %params = @params;
-    my $metric = $params{metric};
-    return ($self->{thresholds}->{$metric}->{warning},
-        $self->{thresholds}->{$metric}->{critical});
-  } else {
-    return ($self->{thresholds}->{default}->{warning},
-        $self->{thresholds}->{default}->{critical});
-  }
-}
-
-sub check_thresholds {
-  my $self = shift;
-  my @params = @_;
-  my $level = $ERRORS{OK};
-  my $warningrange;
-  my $criticalrange;
-  my $value;
-  if (scalar(@params) > 1) {
-    my %params = @params;
-    $value = $params{value};
-    my $metric = $params{metric};
-    if ($metric ne 'default') {
-      $warningrange = exists $self->{thresholds}->{$metric}->{warning} ?
-          $self->{thresholds}->{$metric}->{warning} :
-          $self->{thresholds}->{default}->{warning};
-      $criticalrange = exists $self->{thresholds}->{$metric}->{critical} ?
-          $self->{thresholds}->{$metric}->{critical} :
-          $self->{thresholds}->{default}->{critical};
-    } else {
-      $warningrange = (defined $params{warning}) ?
-          $params{warning} : $self->{thresholds}->{default}->{warning};
-      $criticalrange = (defined $params{critical}) ?
-          $params{critical} : $self->{thresholds}->{default}->{critical};
-    }
-  } else {
-    $value = $params[0];
-    $warningrange = $self->{thresholds}->{default}->{warning};
-    $criticalrange = $self->{thresholds}->{default}->{critical};
-  }
-  if (! defined $warningrange) {
-    # there was no set_thresholds for defaults, no --warning, no --warningx
-  } elsif ($warningrange =~ /^([-+]?[0-9]*\.?[0-9]+)$/) {
-    # warning = 10, warn if > 10 or < 0
-    $level = $ERRORS{WARNING}
-        if ($value > $1 || $value < 0);
-  } elsif ($warningrange =~ /^([-+]?[0-9]*\.?[0-9]+):$/) {
-    # warning = 10:, warn if < 10
-    $level = $ERRORS{WARNING}
-        if ($value < $1);
-  } elsif ($warningrange =~ /^~:([-+]?[0-9]*\.?[0-9]+)$/) {
-    # warning = ~:10, warn if > 10
-    $level = $ERRORS{WARNING}
-        if ($value > $1);
-  } elsif ($warningrange =~ /^([-+]?[0-9]*\.?[0-9]+):([-+]?[0-9]*\.?[0-9]+)$/) {
-    # warning = 10:20, warn if < 10 or > 20
-    $level = $ERRORS{WARNING}
-        if ($value < $1 || $value > $2);
-  } elsif ($warningrange =~ /^@([-+]?[0-9]*\.?[0-9]+):([-+]?[0-9]*\.?[0-9]+)$/) {
-    # warning = @10:20, warn if >= 10 and <= 20
-    $level = $ERRORS{WARNING}
-        if ($value >= $1 && $value <= $2);
-  }
-  if (! defined $criticalrange) {
-    # there was no set_thresholds for defaults, no --critical, no --criticalx
-  } elsif ($criticalrange =~ /^([-+]?[0-9]*\.?[0-9]+)$/) {
-    # critical = 10, crit if > 10 or < 0
-    $level = $ERRORS{CRITICAL}
-        if ($value > $1 || $value < 0);
-  } elsif ($criticalrange =~ /^([-+]?[0-9]*\.?[0-9]+):$/) {
-    # critical = 10:, crit if < 10
-    $level = $ERRORS{CRITICAL}
-        if ($value < $1);
-  } elsif ($criticalrange =~ /^~:([-+]?[0-9]*\.?[0-9]+)$/) {
-    # critical = ~:10, crit if > 10
-    $level = $ERRORS{CRITICAL}
-        if ($value > $1);
-  } elsif ($criticalrange =~ /^([-+]?[0-9]*\.?[0-9]+):([-+]?[0-9]*\.?[0-9]+)$/) {
-    # critical = 10:20, crit if < 10 or > 20
-    $level = $ERRORS{CRITICAL}
-        if ($value < $1 || $value > $2);
-  } elsif ($criticalrange =~ /^@([-+]?[0-9]*\.?[0-9]+):([-+]?[0-9]*\.?[0-9]+)$/) {
-    # critical = @10:20, crit if >= 10 and <= 20
-    $level = $ERRORS{CRITICAL}
-        if ($value >= $1 && $value <= $2);
-  }
-  return $level;
-}
-
-
-package Monitoring::GLPlugin::Commandline::Getopt;
-use strict;
-use File::Basename;
-use Getopt::Long qw(:config no_ignore_case bundling);
-
-# Standard defaults
-my %DEFAULT = (
-  timeout => 15,
-  verbose => 0,
-  license =>
-"This monitoring plugin is free software, and comes with ABSOLUTELY NO WARRANTY.
-It may be used, redistributed and/or modified under the terms of the GNU
-General Public Licence (see http://www.fsf.org/licensing/licenses/gpl.txt).",
-);
-# Standard arguments
-my @ARGS = ({
-    spec => 'usage|?',
-    help => "-?, --usage\n   Print usage information",
-  }, {
-    spec => 'help|h',
-    help => "-h, --help\n   Print detailed help screen",
-  }, {
-    spec => 'version|V',
-    help => "-V, --version\n   Print version information",
-  }, {
-    #spec => 'extra-opts:s@',
-    #help => "--extra-opts=[<section>[@<config_file>]]\n   Section and/or config_file from which to load extra options (may repeat)",
-  }, {
-    spec => 'timeout|t=i',
-    help => sprintf("-t, --timeout=INTEGER\n   Seconds before plugin times out (default: %s)", $DEFAULT{timeout}),
-    default => $DEFAULT{timeout},
-  }, {
-    spec => 'verbose|v+',
-    help => "-v, --verbose\n   Show details for command-line debugging (can repeat up to 3 times)",
-    default => $DEFAULT{verbose},
-  },
-);
-# Standard arguments we traditionally display last in the help output
-my %DEFER_ARGS = map { $_ => 1 } qw(timeout verbose);
-
-sub _init {
-  my $self = shift;
-  my %params = @_;
-  # Check params
-  my %attr = (
-    usage => 1,
-    version => 0,
-    url => 0,
-    plugin => { default => $Monitoring::GLPlugin::pluginname },
-    blurb => 0,
-    extra => 0,
-    'extra-opts' => 0,
-    license => { default => $DEFAULT{license} },
-    timeout => { default => $DEFAULT{timeout} },
-  );
-
-  # Add attr to private _attr hash (except timeout)
-  $self->{timeout} = delete $attr{timeout};
-  $self->{_attr} = { %attr };
-  foreach (keys %{$self->{_attr}}) {
-    if (exists $params{$_}) {
-      $self->{_attr}->{$_} = $params{$_};
-    } else {
-      $self->{_attr}->{$_} = $self->{_attr}->{$_}->{default}
-          if ref ($self->{_attr}->{$_}) eq 'HASH' &&
-              exists $self->{_attr}->{$_}->{default};
-    }
-  }
-  # Chomp _attr values
-  chomp foreach values %{$self->{_attr}};
-
-  # Setup initial args list
-  $self->{_args} = [ grep { exists $_->{spec} } @ARGS ];
-
-  $self
-}
-
-sub new {
-  my $class = shift;
-  my $self = bless {}, $class;
-  $self->_init(@_);
-}
-
-sub add_arg {
-  my $self = shift;
-  my %arg = @_;
-  push (@{$self->{_args}}, \%arg);
-}
-
-sub mod_arg {
-  my $self = shift;
-  my $argname = shift;
-  my %arg = @_;
-  foreach my $old_arg (@{$self->{_args}}) {
-    next unless $old_arg->{spec} =~ /(\w+).*/ && $argname eq $1;
-    foreach my $key (keys %arg) {
-      $old_arg->{$key} = $arg{$key};
-    }
-  }
-}
-
-sub getopts {
-  my $self = shift;
-  my %commandline = ();
-  my @params = map { $_->{spec} } @{$self->{_args}};
-  if (! GetOptions(\%commandline, @params)) {
-    $self->print_help();
-    exit 0;
-  } else {
-    no strict 'refs';
-    no warnings 'redefine';
-    do { $self->print_help(); exit 0; } if $commandline{help};
-    do { $self->print_version(); exit 0 } if $commandline{version};
-    do { $self->print_usage(); exit 3 } if $commandline{usage};
-    foreach (map { $_->{spec} =~ /^([\w\-]+)/; $1; } @{$self->{_args}}) {
-      my $field = $_;
-      *{"$field"} = sub {
-        return $self->{opts}->{$field};
-      };
-    }
-    foreach (map { $_->{spec} =~ /^([\w\-]+)/; $1; }
-        grep { exists $_->{required} && $_->{required} } @{$self->{_args}}) {
-      do { $self->print_usage(); exit 0 } if ! exists $commandline{$_};
-    }
-    foreach (grep { exists $_->{default} } @{$self->{_args}}) {
-      $_->{spec} =~ /^([\w\-]+)/;
-      my $spec = $1;
-      $self->{opts}->{$spec} = $_->{default};
-    }
-    foreach (keys %commandline) {
-      $self->{opts}->{$_} = $commandline{$_};
-    }
-    foreach (grep { exists $_->{env} } @{$self->{_args}}) {
-      $_->{spec} =~ /^([\w\-]+)/;
-      my $spec = $1;
-      if (exists $ENV{'NAGIOS__HOST'.$_->{env}}) {
-        $self->{opts}->{$spec} = $ENV{'NAGIOS__HOST'.$_->{env}};
-      }
-      if (exists $ENV{'NAGIOS__SERVICE'.$_->{env}}) {
-        $self->{opts}->{$spec} = $ENV{'NAGIOS__SERVICE'.$_->{env}};
-      }
-    }
-    foreach (grep { exists $_->{aliasfor} } @{$self->{_args}}) {
-      my $field = $_->{aliasfor};
-      $_->{spec} =~ /^([\w\-]+)/;
-      my $aliasfield = $1;
-      next if $self->{opts}->{$field};
-      *{"$field"} = sub {
-        return $self->{opts}->{$aliasfield};
-      };
-    }
-  }
-}
-
-sub create_opt {
-  my $self = shift;
-  my $key = shift;
-  no strict 'refs';
-  *{"$key"} = sub {
-      return $self->{opts}->{$key};
-  };
-}
-
-sub override_opt {
-  my $self = shift;
-  my $key = shift;
-  my $value = shift;
-  $self->{opts}->{$key} = $value;
-}
-
-sub get {
-  my $self = shift;
-  my $opt = shift;
-  return $self->{opts}->{$opt};
-}
-
-sub print_help {
-  my $self = shift;
-  $self->print_version();
-  printf "\n%s\n", $self->{_attr}->{license};
-  printf "\n%s\n\n", $self->{_attr}->{blurb};
-  $self->print_usage();
-  foreach (grep {
-      ! (exists $_->{hidden} && $_->{hidden}) 
-  } @{$self->{_args}}) {
-    printf " %s\n", $_->{help};
-  }
-  exit 0;
-}
-
-sub print_usage {
-  my $self = shift;
-  printf $self->{_attr}->{usage}, $self->{_attr}->{plugin};
-  print "\n";
-}
-
-sub print_version {
-  my $self = shift;
-  printf "%s %s", $self->{_attr}->{plugin}, $self->{_attr}->{version};
-  printf " [%s]", $self->{_attr}->{url} if $self->{_attr}->{url};
-  print "\n";
-}
-
-sub print_license {
-  my $self = shift;
-  printf "%s\n", $self->{_attr}->{license};
-  print "\n";
-}
-
-
-package Monitoring::GLPlugin::Item;
-our @ISA = qw(Monitoring::GLPlugin);
-
-use strict;
-
-sub new {
-  my $class = shift;
-  my %params = @_;
-  my $self = {
-    blacklisted => 0,
-    info => undef,
-    extendedinfo => undef,
-  };
-  bless $self, $class;
-  $self->init(%params);
-  return $self;
-}
-
-sub check {
-  my $self = shift;
-  my $lists = shift;
-  my @lists = $lists ? @{$lists} : grep { ref($self->{$_}) eq "ARRAY" } keys %{$self};
-  foreach my $list (@lists) {
-    $self->add_info('checking '.$list);
-    foreach my $element (@{$self->{$list}}) {
-      $element->blacklist() if $self->is_blacklisted();
-      $element->check();
-    }
-  }
-}
-
-
-package Monitoring::GLPlugin::TableItem;
-our @ISA = qw(Monitoring::GLPlugin::Item);
-
-use strict;
-
-sub new {
-  my $class = shift;
-  my %params = @_;
-  my $self = {};
-  bless $self, $class;
-  foreach (keys %params) {
-    $self->{$_} = $params{$_};
-  }
-  if ($self->can("finish")) {
-    $self->finish(%params);
-  }
-  return $self;
-}
-
-sub check {
-  my $self = shift;
-  # some tableitems are not checkable, they are only used to enhance other
-  # items (e.g. sensorthresholds enhance sensors)
-  # normal tableitems should have their own check-method
-}
-
-__PACKAGE__
+__END__
